#include <Wire.h>
#include <skPWMServo.h>
#include <SoftwareSerial.h>


// 関数のプロトタイプ宣言
void CtlDev(unsigned char *dt) ;
void DspMon(unsigned char *dt) ;

#define LED_PIN_NO  7              // LED接続ピン番号

SoftwareSerial SoftSerial(2, 3);   // ソフトシリアルライブラリの生成(RX,TX)
skPWMServo     servo(0x40) ;       // PWM/サーボライブラリの生成(I2Cアドレス)

int RcvSts ;                       // 受信の状態フラグ

void setup()
{
     pinMode(LED_PIN_NO,OUTPUT) ;  // LED接続ピンをデジタル出力に設定
     // シリアルの設定
     SoftSerial.begin(9600);       // XBeeの通信で使用
     Serial.begin(9600) ;          // モニターで使用
     Serial.println("RC test with XBee(Receiver)") ;

     // I2C通信の初期化
     Wire.begin() ;
     // サーボシールド(PCA9685)の初期化
     servo.Init() ;
     // PWM出力周波数の設定
     // 50Hz=50.1Hz(19.96ms) 1KHz=1.07KHz(935us)
     servo.SetFrequency(50) ;

     RcvSts = 0 ;

     // 3秒後に開始
     delay(3000);
}
void loop()
{
     unsigned char data[3], rcv ;
     static   int  i, j, kosu ;

     // データを受信するまで待つ
     if (SoftSerial.available()) {
          rcv = SoftSerial.read() ;     // 1バイト読み出す
          if (RcvSts <= 1) {
               // ヘッダの検索とデータ個数の取り出し
               if (RcvSts == 1) {
                    i = j = 0 ;
                    RcvSts = 2 ;
                    kosu   = rcv*3 ;    // データの個数 x 3バイト(1デバイス分)
               }
               if (rcv == 0xFF) {
                    RcvSts = 1 ;        // ヘッダ有りならデータの取り出し開始
               }
          } else {
               i++ ;
               // データの個数 x 3バイト分読み込む
               if (i <= kosu) {
                    data[j] = rcv ;
                    j++ ;
                    // 3バイト毎(1デバイス分)に処理
                    if (j > 2) {
                         j = 0 ;
                         CtlDev(data) ;      // デバイス器機を制御する
                         //DspMon(data) ;      // データ内容を表示する
                    }
                    if (i==kosu) RcvSts = 0 ;// データの取り出し終了
               }
          }
     }
}
// 実際にデバイス器機を動かす処理
void CtlDev(unsigned char *dt)
{
     struct dt_t {
          char ctlno ;
          int  val ;
     } *data ;

     data = (struct dt_t *)dt ;
     // 各デバイス番号(チャンネル番号等)により振り分ける
     switch(data->ctlno) {
        case  0:
        case  1:// サーボモータの処理
                servo.SetDutyRatio(data->ctlno, 0, data->val) ;
                break ;
        case 16:// LEDの点灯処理
                if (data->val==1) digitalWrite(LED_PIN_NO,LOW) ;
                else              digitalWrite(LED_PIN_NO,HIGH) ;
                break ;
        default:
                break ;
     }
}
// データをシリアルモニタに表示させる処理
void DspMon(unsigned char *dt)
{
     struct dt_t {
          char ctlno ;
          int  val ;
     } *data ;

     data = (struct dt_t *)dt ;
     Serial.print(data->ctlno,HEX) ;
     Serial.print(" ") ;
     Serial.println(data->val) ;
}
